@@ -66,6 +66,10 @@ class ThreadPool
     {
       finish();
     }
+    ThreadPool(const ThreadPool &) = delete;
+    ThreadPool &operator=(const ThreadPool &) = delete;
+    ThreadPool(ThreadPool &&) = delete;
+    ThreadPool &operator=(ThreadPool &&) = delete;
 
     /// Queue the callable function \a f for the threads to execute.
     /// A future of the return type of the function is returned to capture the result.
@@ -99,8 +103,8 @@ class ThreadPool
         std::unique_lock<std::mutex> l(m_mutex);
         for(auto&& u : m_finished)
         {
-          unused_variable(u);
-          m_work.push_back({}); // insert empty function object to signal abort
+          (void)u; //unused_variable, to silence the compiler warning about unused variables
+          m_work.emplace_back(); // insert empty function object to signal abort
         }
       }
       m_cond.notify_all();
@@ -108,10 +112,6 @@ class ThreadPool
     }
   private:
 
-    // helper to silence the compiler warning about unused variables
-    template <typename ...Args>
-    void unused_variable(Args&& ...args) { (void)(sizeof...(args)); }
-
     // the work that a worker thread does:
     void threadTask()
     {
